@@ -11,7 +11,7 @@
 #undef  UNDEFID
 #define UNDEFID  CDI_UNDEFID
 
-void basetimeInit(BaseTime *basetime)
+void basetimeInit(basetime_t *basetime)
 {
   if ( basetime == NULL )
     Error("Internal problem! Basetime not allocated.");
@@ -19,6 +19,7 @@ void basetimeInit(BaseTime *basetime)
   (*basetime).ncvarid       = UNDEFID;
   (*basetime).ncdimid       = UNDEFID;
   (*basetime).ncvarboundsid = UNDEFID;
+  (*basetime).leadtimeid    = UNDEFID;
   (*basetime).lwrf          = 0;
 }
 /*
