@@ -116,7 +116,7 @@ void *Pressure(void *argument)
 	{
 	  double *level;
 	  int l;
-	  level = malloc(nlevel*sizeof(double));
+	  level = (double*) malloc(nlevel*sizeof(double));
 	  zaxisInqLevels(zaxisID, level);
 	  for ( l = 0; l < nlevel; l++ )
 	    {
@@ -140,7 +140,7 @@ void *Pressure(void *argument)
 		  nhlevf   = nhlev;
 		  nhlevh   = nhlevf + 1;
 	      
-		  vct = malloc(nvct*sizeof(double));
+		  vct = (double*) malloc(nvct*sizeof(double));
 		  zaxisInqVct(zaxisID, vct);
 		}
 	    }
@@ -154,7 +154,7 @@ void *Pressure(void *argument)
 		  nhlevf   = nhlev - 1;
 		  nhlevh   = nhlev;
 	      
-		  vct = malloc(nvct*sizeof(double));
+		  vct = (double*) malloc(nvct*sizeof(double));
 		  zaxisInqVct(zaxisID, vct);
 		}
 	    }
@@ -166,7 +166,7 @@ void *Pressure(void *argument)
 		  int voff = 4;
 		  double *rvct = NULL;
 
-		  rvct = malloc(nvct*sizeof(double));
+		  rvct = (double*) malloc(nvct*sizeof(double));
 		  zaxisInqVct(zaxisID,rvct);
 
 		  if ( (int)(rvct[0]+0.5) == 100000 && rvct[voff] < rvct[voff+1] )
@@ -178,7 +178,7 @@ void *Pressure(void *argument)
 		      nhlevh   = nhlev + 1;
 
 		      vctsize = 2*nhlevh;
-		      vct = malloc(vctsize*sizeof(double));
+		      vct = (double*) malloc(vctsize*sizeof(double));
 
 		      /* calculate VCT for LM */
 
@@ -213,13 +213,13 @@ void *Pressure(void *argument)
 
   if ( zaxisIDh != -1 && ngp > 0 )
     {
-      ps_prog    = malloc(ngp*sizeof(double));
-      deltap     = malloc(ngp*nhlevf*sizeof(double));
-      full_press = malloc(ngp*nhlevf*sizeof(double));
-      half_press = malloc(ngp*nhlevh*sizeof(double));
+      ps_prog    = (double*) malloc(ngp*sizeof(double));
+      deltap     = (double*) malloc(ngp*nhlevf*sizeof(double));
+      full_press = (double*) malloc(ngp*nhlevf*sizeof(double));
+      half_press = (double*) malloc(ngp*nhlevh*sizeof(double));
     }
   else
-    cdoAbort("No data on hybrid model level found!");
+    cdoAbort("No 3D variable with hybrid sigma pressure coordinate found!");
 
   if ( operatorID == PRESSURE_FL || operatorID == DELTAP )
     zaxisIDp = zaxisCreate(ZAXIS_HYBRID, nhlevf);
@@ -229,7 +229,7 @@ void *Pressure(void *argument)
   {
     double *level;
     int l;
-    level = malloc(nhlevh*sizeof(double));
+    level = (double*) malloc(nhlevh*sizeof(double));
     for ( l = 0; l < nhlevh; l++ ) level[l] = l+1;
     zaxisDefLevels(zaxisIDp, level);
     free(level);
@@ -351,7 +351,7 @@ void *Pressure(void *argument)
     cdoAbort("%s on spectral representation not supported!", var_stdname(surface_air_pressure));
 
   gridsize = gridInqSize(gridID);
-  pdata = malloc(gridsize*sizeof(double));
+  pdata = (double*) malloc(gridsize*sizeof(double));
 
 
   vlistID2 = vlistCreate();
