@@ -44,7 +44,7 @@
 
 
 enum {REMAPCON, REMAPCON2, REMAPBIL, REMAPBIC, REMAPDIS, REMAPNN, REMAPLAF, REMAPSUM,
-      GENCON, GENCON2, GENBIL, GENBIC, GENDIS, GENNN, GENLAF, REMAPXXX, REMAPCONS, GENCONS};
+      GENCON, GENCON2, GENBIL, GENBIC, GENDIS, GENNN, GENLAF, REMAPXXX, REMAPYCON, GENYCON};
 
 enum {HEAP_SORT, MERGE_SORT};
 
@@ -53,9 +53,9 @@ void get_map_type(int operfunc, int *map_type, int *submap_type, int *num_neighb
 {
   switch ( operfunc )
     {
-    case REMAPCONS:
-    case GENCONS:
-      *map_type = MAP_TYPE_CONSPHERE;
+    case REMAPYCON:
+    case GENYCON:
+      *map_type = MAP_TYPE_CONSERV_YAC;
       *remap_order = 1;
       break;
     case REMAPCON:
@@ -111,66 +111,111 @@ int maptype2operfunc(int map_type, int submap_type, int num_neighbors, int remap
       if ( submap_type == SUBMAP_TYPE_LAF )
 	{
 	  operfunc = REMAPLAF;
-	  cdoPrint("Using remaplaf");
+	  // cdoPrint("Using remaplaf");
 	}
       else
 	{
 	  if ( remap_order == 2 )
 	    {
 	      operfunc = REMAPCON2;
-	      cdoPrint("Using remapcon2");
+	      // cdoPrint("Using remapcon2");
 	    }
 	  else
 	    {
 	      operfunc = REMAPCON;
-	      cdoPrint("Using remapcon");
+	      // cdoPrint("Using remapcon");
 	    }
 	}
     }
-  else if ( map_type == MAP_TYPE_CONSPHERE )
+  else if ( map_type == MAP_TYPE_CONSERV_YAC )
     {
-      operfunc = REMAPCONS;
-      cdoPrint("Using remapcons");
+      operfunc = REMAPYCON;
+      // cdoPrint("Using remapycon");
     }
   else if ( map_type == MAP_TYPE_BILINEAR )
     {
       operfunc = REMAPBIL;
-      cdoPrint("Using remapbil");
+      // cdoPrint("Using remapbil");
     }
   else if ( map_type == MAP_TYPE_BICUBIC )
     {
       operfunc = REMAPBIC;
-      cdoPrint("Using remapbic");
+      // cdoPrint("Using remapbic");
     }
   else if ( map_type == MAP_TYPE_DISTWGT )
     {
       if ( num_neighbors == 1 )
 	{
 	  operfunc = REMAPNN;
-	  cdoPrint("Using remapnn");
+	  // cdoPrint("Using remapnn");
 	}
       else
 	{
 	  operfunc = REMAPDIS;
-	  cdoPrint("Using remapdis");
+	  // cdoPrint("Using remapdis");
 	}
     }
   else
     cdoAbort("Unsupported mapping method (map_type = %d)", map_type);
 
   return (operfunc);
+} 
+
+static
+void print_remap_info(int operfunc, remapgrid_t *src_grid, remapgrid_t *tgt_grid, int nmiss)
+{
+  char line[256];
+  char tmpstr[256];
+
+  line[0] = 0;
+
+  if      ( operfunc == REMAPBIL  || operfunc == GENBIL  )  strcpy(line, "SCRIP bilinear");
+  else if ( operfunc == REMAPBIC  || operfunc == GENBIC  )  strcpy(line, "SCRIP bicubic");
+  else if ( operfunc == REMAPNN   || operfunc == GENNN   )  strcpy(line, "SCRIP nearest neighbor");
+  else if ( operfunc == REMAPDIS  || operfunc == GENDIS  )  strcpy(line, "SCRIP distance-weighted average");
+  else if ( operfunc == REMAPCON  || operfunc == GENCON  )  strcpy(line, "SCRIP first order conservative");
+  else if ( operfunc == REMAPCON2 || operfunc == GENCON2 )  strcpy(line, "SCRIP second order conservative");
+  else if ( operfunc == REMAPLAF  || operfunc == GENLAF  )  strcpy(line, "SCRIP largest area fraction");
+  else if ( operfunc == REMAPYCON || operfunc == GENYCON )  strcpy(line, "YAC first order conservative");
+  else                                                      strcpy(line, "Unknown");
+
+  strcat(line, " remapping from ");
+  strcat(line, gridNamePtr(gridInqType(src_grid->gridID)));
+  if ( src_grid->rank == 2 )
+    sprintf(tmpstr, " (%dx%d)", src_grid->dims[0], src_grid->dims[1]);
+  else
+    sprintf(tmpstr, " (%d)", src_grid->dims[0]);
+  strcat(line, tmpstr);
+  strcat(line, " to ");
+  strcat(line, gridNamePtr(gridInqType(tgt_grid->gridID)));
+  if ( tgt_grid->rank == 2 )
+    sprintf(tmpstr, " (%dx%d)", tgt_grid->dims[0], tgt_grid->dims[1]);
+  else
+    sprintf(tmpstr, " (%d)", tgt_grid->dims[0]);
+  strcat(line, tmpstr);
+  strcat(line, " grid");
+
+  if ( nmiss > 0 )
+    {
+      sprintf(tmpstr, ", with source mask (%d)", gridInqSize(src_grid->gridID)-nmiss);
+      strcat(line, tmpstr);
+    }
+
+  cdoPrint(line);
 }
 
+
 double remap_threshhold = 2;
+double remap_search_radius = 180;
 int remap_test = 0;
 int remap_order = 1;
-int remap_store_link_fast = TRUE;
 int remap_non_global = FALSE;
 int remap_num_srch_bins = 180;
 int lremap_num_srch_bins = FALSE;
 int remap_extrapolate = FALSE;
+int remap_genweights = TRUE;
 int lextrapolate = FALSE;
-int max_remaps = 0;
+int max_remaps = -1;
 int sort_mode = HEAP_SORT;
 double remap_area_min = 0;
 
@@ -272,6 +317,26 @@ void get_remap_env(void)
 
   remap_set_threshhold(remap_threshhold);
 
+  envstr = getenv("CDO_REMAP_SEARCH_RADIUS");
+  if ( envstr )
+    {
+      double fval;
+      fval = atof(envstr);
+      if ( fval < 0 || fval > 180 )
+	{
+	  cdoAbort("CDO_REMAP_SEARCH_RADIUS=%g out of bounds (0-180)", fval);
+	}
+      else
+	{
+	  remap_search_radius = fval;
+	  if ( cdoVerbose )
+	    cdoPrint("Set CDO_REMAP_SEARCH_RADIUS to %g", remap_search_radius);
+	}
+    }
+  
+  if ( cdoVerbose )
+    cdoPrint("remap_search_radius = %g", remap_search_radius);
+
   envstr = getenv("REMAP_AREA_MIN");
   if ( envstr )
     {
@@ -312,6 +377,8 @@ void get_remap_env(void)
 	}
     }
 
+  int remap_store_link_fast = TRUE;
+
   envstr = getenv("REMAP_STORE_LINK_FAST");
   if ( envstr )
     {
@@ -354,6 +421,64 @@ void get_remap_env(void)
 	    }
 	}
     }
+
+  envstr = getenv("CDO_REMAP_GENWEIGHTS");
+  if ( envstr )
+    {
+      if ( *envstr )
+	{
+	  if ( memcmp(envstr, "ON", 2) == 0 || memcmp(envstr, "on", 2) == 0 )
+	    {
+	      remap_genweights = TRUE;
+	    }
+	  else if ( memcmp(envstr, "OFF", 3) == 0 || memcmp(envstr, "off", 3) == 0 )
+	    {
+	      remap_genweights = FALSE;
+	    }
+	  else
+	    cdoWarning("Environment variable CDO_REMAP_GENWEIGHTS has wrong value!");
+
+	  if ( cdoVerbose )
+	    {
+	      if ( remap_genweights == TRUE )
+		cdoPrint("Generation of weights enabled!");
+	      else if ( remap_genweights == FALSE )
+		cdoPrint("Generation of weights disabled!");
+	    }
+	}
+    }
+}
+
+static
+void set_halo_to_missval(int nx, int ny, double *array, double missval)
+{
+  int i, j;
+
+  for ( j = 0; j < ny+4; j++ ) array[j*(nx+4)+0]      = missval;
+  for ( j = 0; j < ny+4; j++ ) array[j*(nx+4)+1]      = missval;
+  for ( j = 0; j < ny+4; j++ ) array[j*(nx+4)+nx+2]   = missval;
+  for ( j = 0; j < ny+4; j++ ) array[j*(nx+4)+nx+3]   = missval;
+  for ( i = 0; i < nx+4; i++ ) array[     0*(nx+4)+i] = missval;
+  for ( i = 0; i < nx+4; i++ ) array[     1*(nx+4)+i] = missval;
+  for ( i = 0; i < nx+4; i++ ) array[(ny+2)*(nx+4)+i] = missval;
+  for ( i = 0; i < nx+4; i++ ) array[(ny+3)*(nx+4)+i] = missval;
+}
+
+static
+int is_global_grid(int gridID)
+{
+  int global_grid = TRUE;
+  int non_global = remap_non_global || !gridIsCircular(gridID);
+  int gridtype = gridInqType(gridID);
+
+  if ( (gridtype == GRID_LONLAT && gridIsRotated(gridID)) ||
+       (gridtype == GRID_LONLAT && non_global) ||
+       (gridtype == GRID_LCC) ||
+       (gridtype == GRID_LAEA) ||
+       (gridtype == GRID_SINUSOIDAL) ||
+       (gridtype == GRID_CURVILINEAR && non_global) ) global_grid = FALSE;
+
+  return (global_grid);
 }
 
 static
@@ -380,10 +505,184 @@ void scale_gridbox_area(long gridsize, const double *restrict array1, long grids
     }
 }
 
+static
+int set_remapgrids(int vlistID, int ngrids, int *remapgrids)
+{
+  int index, gridID, gridtype;
+
+  for ( index = 0; index < ngrids; index++ )
+    {
+      remapgrids[index] = TRUE;
+
+      gridID = vlistGrid(vlistID, index);
+      gridtype = gridInqType(gridID);
+
+      if ( gridtype != GRID_LONLAT      &&
+	   gridtype != GRID_GAUSSIAN    &&
+	   gridtype != GRID_LCC         &&
+	   gridtype != GRID_LAEA        &&
+	   gridtype != GRID_SINUSOIDAL  &&
+	   gridtype != GRID_GME         &&
+	   gridtype != GRID_CURVILINEAR &&
+	   gridtype != GRID_UNSTRUCTURED )
+	{
+	  if ( gridtype == GRID_GAUSSIAN_REDUCED )
+	    cdoAbort("Unsupported grid type: %s, use CDO option -R to convert reduced to regular grid!", gridNamePtr(gridtype));
+	  else if ( gridtype == GRID_GENERIC && gridInqSize(gridID) == 1 )
+	    remapgrids[index] = FALSE;
+	  else
+	    cdoAbort("Unsupported grid type: %s", gridNamePtr(gridtype));
+	}
+    }
+
+  for ( index = 0; index < ngrids; index++ )
+    if ( remapgrids[index] == TRUE ) break;
+
+  if ( index == ngrids ) cdoAbort("No remappable grid found!");
+
+  return (index);
+}
+
+static
+int set_max_remaps(int vlistID)
+{
+  int max_remaps = 0;
+  int nzaxis, nvars, index;
+  int zaxisID, zaxissize;
+
+  nzaxis = vlistNzaxis(vlistID);
+  for ( index = 0; index < nzaxis; index++ )
+    {
+      zaxisID = vlistZaxis(vlistID, index);
+      zaxissize = zaxisInqSize(zaxisID);
+      if ( zaxissize > max_remaps ) max_remaps = zaxissize;
+    }
+  
+  nvars = vlistNvars(vlistID);
+  if ( nvars > max_remaps ) max_remaps = nvars;
+
+  max_remaps++;
+
+  if ( cdoVerbose ) cdoPrint("Set max_remaps to %d", max_remaps);
+
+  return (max_remaps);
+}
+
+static
+int get_norm_opt()
+{
+  int norm_opt = NORM_OPT_FRACAREA;
+  char *envstr = getenv("CDO_REMAP_NORMALIZE_OPT");
+
+  if ( envstr && *envstr )
+    {
+      if      ( memcmp(envstr, "frac", 4) == 0 )
+	norm_opt = NORM_OPT_FRACAREA;
+      else if ( memcmp(envstr, "dest", 4) == 0 )
+	norm_opt = NORM_OPT_DESTAREA;
+      else if ( memcmp(envstr, "none", 4) == 0 )
+	norm_opt = NORM_OPT_NONE;
+      else
+	cdoWarning("CDO_REMAP_NORMALIZE_OPT=%s unsupported!", envstr);
+    }
+
+  if ( cdoVerbose )
+    {
+      if ( norm_opt == NORM_OPT_FRACAREA )
+	cdoPrint("Normalization option: frac");
+      else if ( norm_opt == NORM_OPT_DESTAREA )
+	cdoPrint("Normalization option: dest");
+      else
+	cdoPrint("Normalization option: none");
+    }
+
+  return (norm_opt);
+}
+
+static
+void remap_normalize(int norm_opt, int gridsize, double *array, double missval, remapgrid_t *tgt_grid)
+{
+  int i;
+  double grid_err;
+
+  if ( norm_opt == NORM_OPT_NONE )
+    {
+      for ( i = 0; i < gridsize; i++ )
+	{
+	  if ( !DBL_IS_EQUAL(array[i], missval) )
+	    {
+	      grid_err = tgt_grid->cell_frac[i]*tgt_grid->cell_area[i];
+	      if ( fabs(grid_err) > 0 )
+		array[i] = array[i]/grid_err;
+	      else
+		array[i] = missval;
+	    }
+	}
+    }
+  else if ( norm_opt == NORM_OPT_DESTAREA )
+    {
+      for ( i = 0; i < gridsize; i++ )
+	{
+	  if ( !DBL_IS_EQUAL(array[i], missval) )
+	    {
+	      if ( fabs(tgt_grid->cell_frac[i]) > 0 )
+		array[i] = array[i]/tgt_grid->cell_frac[i];
+	      else
+		array[i] = missval;
+	    }
+	}
+    }
+
+  if ( remap_area_min > 0 )
+    {
+      for ( i = 0; i < gridsize; i++ )
+	{
+	  //printf("%d %g %g\n", i, remaps[r].tgt_grid.cell_frac[i], remaps[r].tgt_grid.cell_area[i]);
+	  if ( tgt_grid->cell_frac[i] < remap_area_min ) array[i] = missval;
+	}
+    }
+}
+
 
 int timer_remap, timer_remap_init, timer_remap_sort;
 int timer_remap_bil, timer_remap_nn, timer_remap_con, timer_remap_con_l1, timer_remap_con_l2;
 
+static
+void init_remap_timer(void)
+{
+  timer_remap        = timer_new("remap");
+  timer_remap_init   = timer_new("remap init");
+  timer_remap_sort   = timer_new("remap sort");
+  timer_remap_bil    = timer_new("remap bil");
+  timer_remap_nn     = timer_new("remap nn");
+  timer_remap_con    = timer_new("remap con");
+  timer_remap_con_l1 = timer_new("remap con loop1");
+  timer_remap_con_l2 = timer_new("remap con loop2");
+}
+
+static
+void sort_remap_add(remapvars_t *remapvars)
+{
+  if ( cdoTimer ) timer_start(timer_remap_sort);
+  if ( sort_mode == MERGE_SORT )
+    { /* 
+      ** use a combination of the old sort_add and a split and merge approach.
+      ** The chunk size is determined by MERGE_SORT_LIMIT_SIZE in remaplib.c. 
+      ** OpenMP parallelism is supported
+      */   
+      sort_iter(remapvars->num_links, remapvars->num_wts,
+		remapvars->tgt_grid_add, remapvars->src_grid_add,
+		remapvars->wts, ompNumThreads);
+    }
+  else
+    { /* use a pure heap sort without any support of parallelism */
+      sort_add(remapvars->num_links, remapvars->num_wts,
+	       remapvars->tgt_grid_add, remapvars->src_grid_add,
+	       remapvars->wts);
+    }
+  if ( cdoTimer ) timer_stop(timer_remap_sort);
+}
+
 
 void *Remap(void *argument)
 {
@@ -391,16 +690,13 @@ void *Remap(void *argument)
   int operfunc;
   int streamID1, streamID2 = -1;
   int nrecs, ngrids;
-  int nzaxis, zaxisID, zaxissize;
-  int nvars;
   int index;
   int tsID, recID, varID, levelID;
   int gridsize, gridsize2;
   int vlistID1, vlistID2;
   int taxisID1, taxisID2;
   int gridID1 = -1, gridID2;
-  int gridtype;
-  int nmiss1, nmiss2, i, j, r;
+  int nmiss1, nmiss2, i, j, r = -1;
   int *imask = NULL;
   int nremaps = 0;
   int norm_opt = NORM_OPT_NONE;
@@ -408,29 +704,17 @@ void *Remap(void *argument)
   int submap_type = SUBMAP_TYPE_NONE;
   int num_neighbors = 4;
   int need_gradiants = FALSE;
-  int non_global;
   int grid1sizemax;
-  short *remapgrids = NULL;
+  int *remapgrids = NULL;
   char varname[CDI_MAX_NAME];
   double missval;
   double *array1 = NULL, *array2 = NULL;
   double *grad1_lat = NULL, *grad1_lon = NULL, *grad1_latlon = NULL;
-  remap_t *remaps;
-  char *envstr;
+  remap_t *remaps = NULL;
   char *remap_file = NULL;
   int lwrite_remap;
 
-  if ( cdoTimer )
-    {
-      timer_remap        = timer_new("remap");
-      timer_remap_init   = timer_new("remap init");
-      timer_remap_sort   = timer_new("remap sort");
-      timer_remap_bil    = timer_new("remap bil");
-      timer_remap_nn     = timer_new("remap nn");
-      timer_remap_con    = timer_new("remap con");
-      timer_remap_con_l1 = timer_new("remap con loop1");
-      timer_remap_con_l2 = timer_new("remap con loop2");
-    }
+  if ( cdoTimer ) init_remap_timer();
 
   cdoInitialize(argument);
 
@@ -450,8 +734,8 @@ void *Remap(void *argument)
   cdoOperatorAdd("gennn",        GENNN,        1, NULL);
   cdoOperatorAdd("genlaf",       GENLAF,       1, NULL);
   cdoOperatorAdd("remap",        REMAPXXX,     0, NULL);
-  cdoOperatorAdd("remapcons",    REMAPCONS,    0, NULL);
-  cdoOperatorAdd("gencons",      GENCONS,      1, NULL);
+  cdoOperatorAdd("remapycon",    REMAPYCON,    0, NULL);
+  cdoOperatorAdd("genycon",      GENYCON,      1, NULL);
 
   operatorID   = cdoOperatorID();
   operfunc     = cdoOperatorF1(operatorID);
@@ -475,7 +759,7 @@ void *Remap(void *argument)
 
   if ( operfunc == REMAPXXX )
     {
-      operatorInputArg("grid description file or name, remap file (SCRIP netCDF)");
+      operatorInputArg("grid description file or name, remap weights file (SCRIP netCDF)");
       operatorCheckArgc(2);
       gridID2 = cdoDefineGrid(operatorArgv()[0]);
       remap_file = operatorArgv()[1];
@@ -497,70 +781,29 @@ void *Remap(void *argument)
   vlistDefTaxis(vlistID2, taxisID2);
 
   ngrids = vlistNgrids(vlistID1);
-  remapgrids = malloc(ngrids*sizeof(short));
-  for ( index = 0; index < ngrids; index++ )
-    {
-      remapgrids[index] = TRUE;
-
-      gridID1 = vlistGrid(vlistID1, index);
-      gridtype = gridInqType(gridID1);
-
-      if ( gridtype != GRID_LONLAT      &&
-	   gridtype != GRID_GAUSSIAN    &&
-	   gridtype != GRID_LCC         &&
-	   gridtype != GRID_LAEA        &&
-	   gridtype != GRID_SINUSOIDAL  &&
-	   gridtype != GRID_GME         &&
-	   gridtype != GRID_CURVILINEAR &&
-	   gridtype != GRID_UNSTRUCTURED )
-	{
-	  if ( gridInqType(gridID1) == GRID_GAUSSIAN_REDUCED )
-	    cdoAbort("Unsupported grid type: %s, use CDO option -R to convert reduced to regular grid!",
-		     gridNamePtr(gridInqType(gridID1)));
-	  else if ( gridInqType(gridID1) == GRID_GENERIC && gridInqSize(gridID1) == 1 )
-	    remapgrids[index] = FALSE;
-	  else
-	    cdoAbort("Unsupported grid type: %s", gridNamePtr(gridInqType(gridID1)));
-	}
-
-      if ( remapgrids[index] )
-	vlistChangeGridIndex(vlistID2, index, gridID2);
-    }
-
-  for ( index = 0; index < ngrids; index++ )
-    if ( remapgrids[index] == TRUE ) break;
-
-  if ( index == ngrids )
-    cdoAbort("No remappable grid found!");
-
+  remapgrids = (int*) malloc(ngrids*sizeof(int));
+  index = set_remapgrids(vlistID1, ngrids, remapgrids);
   gridID1 = vlistGrid(vlistID1, index);
 
-  if ( max_remaps == 0 )
+  for ( index = 0; index < ngrids; index++ )
+    if ( remapgrids[index] )
+      vlistChangeGridIndex(vlistID2, index, gridID2);
+
+  if ( max_remaps == -1 ) max_remaps = set_max_remaps(vlistID1);
+
+  if ( max_remaps > 0 )
     {
-      nzaxis = vlistNzaxis(vlistID1);
-      for ( index = 0; index < nzaxis; index++ )
-        {
-	  zaxisID = vlistZaxis(vlistID1, index);
-	  zaxissize = zaxisInqSize(zaxisID);
-          if ( zaxissize > max_remaps ) max_remaps = zaxissize;
+      remaps = (remap_t*) malloc(max_remaps*sizeof(remap_t));
+      for ( r = 0; r < max_remaps; r++ )
+	{
+	  remaps[r].gridID   = -1;
+	  remaps[r].gridsize = 0;
+	  remaps[r].nmiss    = 0;
 	}
-
-      nvars = vlistNvars(vlistID1);
-      if ( nvars > max_remaps ) max_remaps = nvars;
-
-      max_remaps++;
-
-      if ( cdoVerbose )
-        cdoPrint("Set max_remaps to %d", max_remaps);
     }
 
-  remaps = malloc(max_remaps*sizeof(remap_t));
-  for ( r = 0; r < max_remaps; r++ )
-    {
-      remaps[r].gridID   = -1;
-      remaps[r].gridsize = 0;
-      remaps[r].nmiss    = 0;
-    }
+  if ( lwrite_remap || operfunc == REMAPXXX )
+    remap_genweights = TRUE;
 
   if ( operfunc == REMAPXXX )
     {
@@ -577,15 +820,7 @@ void *Remap(void *argument)
       if ( map_type == MAP_TYPE_DISTWGT && !lextrapolate ) remap_extrapolate = TRUE;
       if ( gridIsCircular(gridID1)      && !lextrapolate ) remap_extrapolate = TRUE;
 
-      non_global = remap_non_global || !gridIsCircular(gridID1);
-      if ( !remap_extrapolate && gridInqSize(gridID1) > 1 &&
-	   map_type == MAP_TYPE_DISTWGT &&
-	   ((gridInqType(gridID1) == GRID_LONLAT && gridIsRotated(gridID1)) ||
-	    (gridInqType(gridID1) == GRID_LONLAT && non_global) ||
-	    (gridInqType(gridID1) == GRID_LCC) ||
-	    (gridInqType(gridID1) == GRID_LAEA) ||
-	    (gridInqType(gridID1) == GRID_SINUSOIDAL) ||
-	    (gridInqType(gridID1) == GRID_CURVILINEAR && non_global)) )
+      if ( map_type == MAP_TYPE_DISTWGT && !remap_extrapolate && gridInqSize(gridID1) > 1 &&  !is_global_grid(gridID1) )
 	{
 	  remaps[0].gridsize += 4*(gridInqXsize(gridID1)+2) + 4*(gridInqYsize(gridID1)+2);
 	  remaps[0].src_grid.non_global = TRUE;
@@ -608,7 +843,7 @@ void *Remap(void *argument)
 	  int gridID2_gme;
 	  int isize = 0;
 	  remaps[0].tgt_grid.nvgp = gridInqSize(gridID2);
-	  remaps[0].tgt_grid.vgpm = realloc(remaps[0].tgt_grid.vgpm, gridInqSize(gridID2)*sizeof(int));
+	  remaps[0].tgt_grid.vgpm = (int*) realloc(remaps[0].tgt_grid.vgpm, gridInqSize(gridID2)*sizeof(int));
 	  gridID2_gme = gridToUnstructured(gridID2, 1);
 	  gridInqMaskGME(gridID2_gme, remaps[0].tgt_grid.vgpm);
 	  for ( i = 0; i < gridsize2; ++i )
@@ -625,38 +860,16 @@ void *Remap(void *argument)
 
       if ( remap_test ) reorder_links(&remaps[0].vars);
     }
-
-  get_map_type(operfunc, &map_type, &submap_type, &num_neighbors, &remap_order);
-
-  if ( map_type == MAP_TYPE_CONSERV ||map_type == MAP_TYPE_CONSPHERE )
+  else
     {
-      norm_opt = NORM_OPT_FRACAREA;
-
-      envstr = getenv("NORMALIZE_OPT");
-
-      if ( envstr && *envstr )
-        {
-	  if      ( memcmp(envstr, "frac", 4) == 0 )
-	    norm_opt = NORM_OPT_FRACAREA;
-	  else if ( memcmp(envstr, "dest", 4) == 0 )
-	    norm_opt = NORM_OPT_DESTAREA;
-	  else if ( memcmp(envstr, "none", 4) == 0 )
-	    norm_opt = NORM_OPT_NONE;
-	  else
-	    cdoWarning("NORMALIZE_OPT=%s unsupported!", envstr);
-	}
-
-      if ( cdoVerbose )
-        {
-	  if ( norm_opt == NORM_OPT_FRACAREA )
-	    cdoPrint("Normalization option: frac");
-	  else if ( norm_opt == NORM_OPT_DESTAREA )
-	    cdoPrint("Normalization option: dest");
-	  else
-	    cdoPrint("Normalization option: none");
-	}
+      get_map_type(operfunc, &map_type, &submap_type, &num_neighbors, &remap_order);
     }
 
+  if ( remap_genweights == FALSE && map_type != MAP_TYPE_BILINEAR && map_type != MAP_TYPE_BICUBIC && map_type != MAP_TYPE_CONSERV_YAC )
+    remap_genweights = TRUE;
+
+  if ( map_type == MAP_TYPE_CONSERV || map_type == MAP_TYPE_CONSERV_YAC ) norm_opt = get_norm_opt();
+
   grid1sizemax = vlistGridsizeMax(vlistID1);
 
   if ( map_type == MAP_TYPE_BICUBIC ) need_gradiants = TRUE;
@@ -670,21 +883,20 @@ void *Remap(void *argument)
 
   if ( need_gradiants )
     {
-      grad1_lat    = malloc(grid1sizemax*sizeof(double));
-      grad1_lon    = malloc(grid1sizemax*sizeof(double));
-      grad1_latlon = malloc(grid1sizemax*sizeof(double));
+      grad1_lat    = (double*) malloc(grid1sizemax*sizeof(double));
+      grad1_lon    = (double*) malloc(grid1sizemax*sizeof(double));
+      grad1_latlon = (double*) malloc(grid1sizemax*sizeof(double));
     }
 
-  array1 = malloc(grid1sizemax*sizeof(double));
-  imask  = malloc(grid1sizemax*sizeof(int));
+  array1 = (double*) malloc(grid1sizemax*sizeof(double));
+  imask  = (int*) malloc(grid1sizemax*sizeof(int));
 
   gridsize = gridInqSize(gridID2);
-  array2   = malloc(gridsize*sizeof(double));
+  array2   = (double*) malloc(gridsize*sizeof(double));
 
   if ( ! lwrite_remap )
     {
       streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
-
       streamDefVlist(streamID2, vlistID2);
     }
 
@@ -714,7 +926,7 @@ void *Remap(void *argument)
 		}
 	    }
 
-	  if ( map_type != MAP_TYPE_CONSERV && map_type != MAP_TYPE_CONSPHERE && 
+	  if ( map_type != MAP_TYPE_CONSERV && map_type != MAP_TYPE_CONSERV_YAC && 
 	       gridInqType(gridID1) == GRID_GME && gridInqType(gridID2) == GRID_GME )
 	    cdoAbort("Only conservative remapping is available to remap between GME grids!");
 	  /*
@@ -725,15 +937,7 @@ void *Remap(void *argument)
 	  gridsize = gridInqSize(gridID1);
 
 	  if ( gridIsCircular(gridID1) && !lextrapolate ) remap_extrapolate = TRUE;
-	  non_global = remap_non_global || !gridIsCircular(gridID1);
-	  if ( !remap_extrapolate && gridInqSize(gridID1) > 1 &&
-	       map_type == MAP_TYPE_DISTWGT  &&
-	       ((gridInqType(gridID1) == GRID_LONLAT && gridIsRotated(gridID1)) ||
-		(gridInqType(gridID1) == GRID_LONLAT && non_global) ||
-		(gridInqType(gridID1) == GRID_LCC) ||
-		(gridInqType(gridID1) == GRID_LAEA) ||
-		(gridInqType(gridID1) == GRID_SINUSOIDAL) ||
-		(gridInqType(gridID1) == GRID_CURVILINEAR && non_global)) )
+	  if ( map_type == MAP_TYPE_DISTWGT && !remap_extrapolate && gridInqSize(gridID1) > 1 && !is_global_grid(gridID1) )
 	    {
 	      int gridsize_new;
 	      int nx, ny;
@@ -743,14 +947,14 @@ void *Remap(void *argument)
 	      if ( gridsize_new > grid1sizemax )
 		{
 		  grid1sizemax = gridsize_new;
-		  array1 = realloc(array1, grid1sizemax*sizeof(double));
-		  imask  = realloc(imask, grid1sizemax*sizeof(int));
+		  array1 = (double*) realloc(array1, grid1sizemax*sizeof(double));
+		  imask  = (int*) realloc(imask, grid1sizemax*sizeof(int));
 
 		  if ( need_gradiants )
 		    {
-		      grad1_lat    = realloc(grad1_lat, grid1sizemax*sizeof(double));
-		      grad1_lon    = realloc(grad1_lon, grid1sizemax*sizeof(double));
-		      grad1_latlon = realloc(grad1_latlon, grid1sizemax*sizeof(double));
+		      grad1_lat    = (double*) realloc(grad1_lat, grid1sizemax*sizeof(double));
+		      grad1_lon    = (double*) realloc(grad1_lon, grid1sizemax*sizeof(double));
+		      grad1_latlon = (double*) realloc(grad1_latlon, grid1sizemax*sizeof(double));
 		    }
 		}
 	      
@@ -758,14 +962,7 @@ void *Remap(void *argument)
 		for ( i = nx-1; i >= 0; i-- )
 		  array1[(j+2)*(nx+4)+i+2] = array1[j*nx+i];
 
-	      for ( j = 0; j < ny+4; j++ ) array1[j*(nx+4)+0]      = missval;
-	      for ( j = 0; j < ny+4; j++ ) array1[j*(nx+4)+1]      = missval;
-	      for ( j = 0; j < ny+4; j++ ) array1[j*(nx+4)+nx+2]   = missval;
-	      for ( j = 0; j < ny+4; j++ ) array1[j*(nx+4)+nx+3]   = missval;
-	      for ( i = 0; i < nx+4; i++ ) array1[     0*(nx+4)+i] = missval;
-	      for ( i = 0; i < nx+4; i++ ) array1[     1*(nx+4)+i] = missval;
-	      for ( i = 0; i < nx+4; i++ ) array1[(ny+2)*(nx+4)+i] = missval;
-	      for ( i = 0; i < nx+4; i++ ) array1[(ny+3)*(nx+4)+i] = missval;
+	      set_halo_to_missval(nx, ny, array1, missval);
 
 	      gridsize = gridsize_new;
 	      nmiss1 += 4*(nx+2) + 4*(ny+2);
@@ -804,15 +1001,7 @@ void *Remap(void *argument)
 		{
 		  if ( gridIsCircular(gridID1) && !lextrapolate ) remap_extrapolate = TRUE;
 		  remaps[r].src_grid.non_global = FALSE;
-		  non_global = remap_non_global || !gridIsCircular(gridID1);
-		  if ( !remap_extrapolate && gridInqSize(gridID1) > 1 &&
-		       map_type == MAP_TYPE_DISTWGT &&
-		       ((gridInqType(gridID1) == GRID_LONLAT && gridIsRotated(gridID1)) ||
-			(gridInqType(gridID1) == GRID_LONLAT && non_global) ||
-			(gridInqType(gridID1) == GRID_LCC) ||
-			(gridInqType(gridID1) == GRID_LAEA) ||
-			(gridInqType(gridID1) == GRID_SINUSOIDAL) ||
-			(gridInqType(gridID1) == GRID_CURVILINEAR && non_global)) )
+		  if ( map_type == MAP_TYPE_DISTWGT && !remap_extrapolate && gridInqSize(gridID1) > 1 && !is_global_grid(gridID1) )
 		    {
 		      remaps[r].src_grid.non_global = TRUE;
 		    }
@@ -863,7 +1052,7 @@ void *Remap(void *argument)
 
 	      memcpy(remaps[r].src_grid.mask, imask, remaps[r].src_grid.size*sizeof(int));
 
-	      if ( map_type == MAP_TYPE_CONSERV || map_type == MAP_TYPE_CONSPHERE )
+	      if ( map_type == MAP_TYPE_CONSERV || map_type == MAP_TYPE_CONSERV_YAC )
 		{
 		  memset(remaps[r].src_grid.cell_area, 0, remaps[r].src_grid.size*sizeof(double));
 		  memset(remaps[r].src_grid.cell_frac, 0, remaps[r].src_grid.size*sizeof(double));
@@ -876,40 +1065,25 @@ void *Remap(void *argument)
 	      remap_vars_init(map_type, remaps[r].src_grid.size, remaps[r].tgt_grid.size, &remaps[r].vars);
 	      if ( cdoTimer ) timer_stop(timer_remap_init);
 
-	      if      ( map_type == MAP_TYPE_CONSERV   ) remap_conserv(&remaps[r].src_grid, &remaps[r].tgt_grid, &remaps[r].vars);
-	      else if ( map_type == MAP_TYPE_BILINEAR  ) remap_bilin(&remaps[r].src_grid, &remaps[r].tgt_grid, &remaps[r].vars);
-	      else if ( map_type == MAP_TYPE_BICUBIC   ) remap_bicub(&remaps[r].src_grid, &remaps[r].tgt_grid, &remaps[r].vars);
-	      else if ( map_type == MAP_TYPE_DISTWGT   ) remap_distwgt(num_neighbors, &remaps[r].src_grid, &remaps[r].tgt_grid, &remaps[r].vars);
-	      else if ( map_type == MAP_TYPE_CONSPHERE ) remap_consphere(&remaps[r].src_grid, &remaps[r].tgt_grid, &remaps[r].vars);
-
-	      if ( remaps[r].vars.num_links != remaps[r].vars.max_links )
-		resize_remap_vars(&remaps[r].vars, remaps[r].vars.num_links-remaps[r].vars.max_links);
-
-	      if ( remaps[r].vars.sort_add )
+	      if ( remap_genweights )
 		{
-		  if ( cdoTimer ) timer_start(timer_remap_sort);
-		  if ( sort_mode == MERGE_SORT )
-		    { /* 
-		      ** use a combination of the old sort_add and a split and merge approach.
-		      ** The chunk size is determined by MERGE_SORT_LIMIT_SIZE in remaplib.c. 
-		      ** OpenMP parallelism is supported
-		      */   
-		      sort_iter(remaps[r].vars.num_links, remaps[r].vars.num_wts,
-				remaps[r].vars.tgt_grid_add, remaps[r].vars.src_grid_add,
-				remaps[r].vars.wts, ompNumThreads);
-		    }
-		  else
-		    { /* use a pure heap sort without any support of parallelism */
-		      sort_add(remaps[r].vars.num_links, remaps[r].vars.num_wts,
-			       remaps[r].vars.tgt_grid_add, remaps[r].vars.src_grid_add,
-			       remaps[r].vars.wts);
-		    }
-		  if ( cdoTimer ) timer_stop(timer_remap_sort);
+		  print_remap_info(operfunc, &remaps[r].src_grid, &remaps[r].tgt_grid, nmiss1);
+
+		  if      ( map_type == MAP_TYPE_CONSERV   ) scrip_remap_weights_conserv(&remaps[r].src_grid, &remaps[r].tgt_grid, &remaps[r].vars);
+		  else if ( map_type == MAP_TYPE_BILINEAR  ) scrip_remap_weights_bilinear(&remaps[r].src_grid, &remaps[r].tgt_grid, &remaps[r].vars);
+		  else if ( map_type == MAP_TYPE_BICUBIC   ) scrip_remap_weights_bicubic(&remaps[r].src_grid, &remaps[r].tgt_grid, &remaps[r].vars);
+		  else if ( map_type == MAP_TYPE_DISTWGT   ) scrip_remap_weights_distwgt(num_neighbors, &remaps[r].src_grid, &remaps[r].tgt_grid, &remaps[r].vars);
+		  else if ( map_type == MAP_TYPE_CONSERV_YAC ) remap_weights_conserv(&remaps[r].src_grid, &remaps[r].tgt_grid, &remaps[r].vars);
+
+		  if ( remaps[r].vars.num_links != remaps[r].vars.max_links )
+		    resize_remap_vars(&remaps[r].vars, remaps[r].vars.num_links-remaps[r].vars.max_links);
+		  
+		  if ( remaps[r].vars.sort_add ) sort_remap_add(&remaps[r].vars);
+
+		  if ( lwrite_remap ) goto WRITE_REMAP;
+
+		  if ( remap_test ) reorder_links(&remaps[r].vars);
 		}
-
-	      if ( lwrite_remap ) goto WRITE_REMAP;
-
-	      if ( remap_test ) reorder_links(&remaps[r].vars);
 	    }
 
 	  if ( gridInqType(gridID1) == GRID_GME )
@@ -919,63 +1093,39 @@ void *Remap(void *argument)
 		if ( remaps[r].src_grid.vgpm[i] ) array1[j++] = array1[i];
 	    }
 	  
-	  if ( need_gradiants )
+	  if ( remap_genweights )
 	    {
-	      if ( remaps[r].src_grid.rank != 2 && remap_order == 2 )
-		cdoAbort("Second order remapping is only available for 2D grids!");
+	      if ( need_gradiants )
+		{
+		  if ( remaps[r].src_grid.rank != 2 && remap_order == 2 )
+		    cdoAbort("Second order remapping is not only available for unstructured grids!");
 
-	      remap_gradients(remaps[r].src_grid, array1, grad1_lat, grad1_lon, grad1_latlon);
+		  remap_gradients(remaps[r].src_grid, array1, grad1_lat, grad1_lon, grad1_latlon);
+		}
+
+	      if ( operfunc == REMAPLAF )
+		remap_laf(array2, missval, gridInqSize(gridID2), remaps[r].vars.num_links, remaps[r].vars.wts,
+			  remaps[r].vars.num_wts, remaps[r].vars.tgt_grid_add, remaps[r].vars.src_grid_add, array1);
+	      else if ( operfunc == REMAPSUM )
+		remap_sum(array2, missval, gridInqSize(gridID2), remaps[r].vars.num_links, remaps[r].vars.wts,
+			  remaps[r].vars.num_wts, remaps[r].vars.tgt_grid_add, remaps[r].vars.src_grid_add, array1);
+	      else
+		remap(array2, missval, gridInqSize(gridID2), remaps[r].vars.num_links, remaps[r].vars.wts,
+		      remaps[r].vars.num_wts, remaps[r].vars.tgt_grid_add, remaps[r].vars.src_grid_add,
+		      array1, grad1_lat, grad1_lon, grad1_latlon, remaps[r].vars.links);
 	    }
-
-	  if ( operfunc == REMAPLAF )
-	    remap_laf(array2, missval, gridInqSize(gridID2), remaps[r].vars.num_links, remaps[r].vars.wts,
-		  remaps[r].vars.num_wts, remaps[r].vars.tgt_grid_add, remaps[r].vars.src_grid_add, array1);
-	  else if ( operfunc == REMAPSUM )
-	    remap_sum(array2, missval, gridInqSize(gridID2), remaps[r].vars.num_links, remaps[r].vars.wts,
-		  remaps[r].vars.num_wts, remaps[r].vars.tgt_grid_add, remaps[r].vars.src_grid_add, array1);
 	  else
-	    remap(array2, missval, gridInqSize(gridID2), remaps[r].vars.num_links, remaps[r].vars.wts,
-		  remaps[r].vars.num_wts, remaps[r].vars.tgt_grid_add, remaps[r].vars.src_grid_add,
-		  array1, grad1_lat, grad1_lon, grad1_latlon, remaps[r].vars.links);
+	    {
+	      if      ( map_type == MAP_TYPE_BILINEAR    ) scrip_remap_bilinear(&remaps[r].src_grid, &remaps[r].tgt_grid, array1, array2, missval);
+	      else if ( map_type == MAP_TYPE_BICUBIC     ) scrip_remap_bicubic(&remaps[r].src_grid, &remaps[r].tgt_grid, array1, array2, missval);
+	      else if ( map_type == MAP_TYPE_CONSERV_YAC ) remap_conserv(&remaps[r].src_grid, &remaps[r].tgt_grid, array1, array2, missval);
+	    }
 
 	  gridsize2 = gridInqSize(gridID2);
 
 	  /* used only to check the result of remapcon */
-	  if ( operfunc == REMAPCON || operfunc == REMAPCON2|| operfunc == REMAPCONS )
-	    {
-	      double grid2_err;
-
-	      if ( remaps[r].vars.norm_opt == NORM_OPT_NONE )
-		{
-		  for ( i = 0; i < gridsize2; i++ )
-		    {
-		      grid2_err = remaps[r].tgt_grid.cell_frac[i]*remaps[r].tgt_grid.cell_area[i];
-		      if ( fabs(grid2_err) > 0 )
-			array2[i] = array2[i]/grid2_err;
-		      else
-			array2[i] = missval;
-		    }
-		}
-	      else if ( remaps[r].vars.norm_opt == NORM_OPT_DESTAREA )
-		{
-		  for ( i = 0; i < gridsize2; i++ )
-		    {
-		      if ( fabs(remaps[r].tgt_grid.cell_frac[i]) > 0 )
-			array2[i] = array2[i]/remaps[r].tgt_grid.cell_frac[i];
-		      else
-			array2[i] = missval;
-		    }
-		}
-
-	      if ( remap_area_min > 0 )
-		{
-		  for ( i = 0; i < gridsize2; i++ )
-		    {
-		      //printf("%d %g %g\n", i, remaps[r].tgt_grid.cell_frac[i], remaps[r].tgt_grid.cell_area[i]);
-		      if ( remaps[r].tgt_grid.cell_frac[i] < remap_area_min ) array2[i] = missval;
-		    }
-		}
-	    }
+	  if ( operfunc == REMAPCON || operfunc == REMAPCON2 || operfunc == REMAPYCON )
+	    remap_normalize(remaps[r].vars.norm_opt, gridsize2, array2, missval, &remaps[r].tgt_grid);
 
 	  if ( operfunc == REMAPSUM )
 	    {
@@ -996,7 +1146,7 @@ void *Remap(void *argument)
 	    }
 
 	  vlistInqVarName(vlistID1, varID, varname);
-	  if ( operfunc == REMAPCON || operfunc == REMAPCON2 || operfunc == REMAPCONS )
+	  if ( operfunc == REMAPCON || operfunc == REMAPCON2 || operfunc == REMAPYCON )
 	    if ( strcmp(varname, "gridbox_area") == 0 )
 	      {
 		scale_gridbox_area(gridsize, array1, gridsize2, array2, remaps[r].tgt_grid.cell_area);
@@ -1050,15 +1200,18 @@ void *Remap(void *argument)
   if ( grad1_lon ) free(grad1_lon);
   if ( grad1_lat ) free(grad1_lat);
 
-  for ( r = 0; r < nremaps; r++ )
+  if ( max_remaps > 0 )
     {
-      remapVarsFree(&remaps[r].vars);
-      remapGridFree(&remaps[r].src_grid);
-      remapGridFree(&remaps[r].tgt_grid);
+      for ( r = 0; r < nremaps; r++ )
+	{
+	  remapVarsFree(&remaps[r].vars);
+	  remapGridFree(&remaps[r].src_grid);
+	  remapGridFree(&remaps[r].tgt_grid);
+	}
+      
+      if ( remaps ) free(remaps);
     }
 
-  if ( remaps ) free(remaps);
-
   cdoFinish();
 
   return (0);
