@@ -112,7 +112,7 @@ void *Replacevalues(void *argument)
 
   gridsize = vlistGridsizeMax(vlistID1);
 
-  array = malloc(gridsize*sizeof(double));
+  array = (double*) malloc(gridsize*sizeof(double));
 
   tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
