@@ -134,12 +134,12 @@ void *Yhourstat(void *argument)
   nvars    = vlistNvars(vlistID1);
   nrecords = vlistNrecs(vlistID1);
 
-  recVarID   = malloc(nrecords*sizeof(int));
-  recLevelID = malloc(nrecords*sizeof(int));
+  recVarID   = (int*) malloc(nrecords*sizeof(int));
+  recLevelID = (int*) malloc(nrecords*sizeof(int));
 
   gridsize = vlistGridsizeMax(vlistID1);
   field_init(&field);
-  field.ptr = malloc(gridsize*sizeof(double));
+  field.ptr = (double*) malloc(gridsize*sizeof(double));
 
   tsID = 0;
   otsID = 0;
@@ -183,7 +183,7 @@ void *Yhourstat(void *argument)
 	      if ( nmiss > 0 || samp1[houroy][varID][levelID].ptr )
 		{
 		  if ( samp1[houroy][varID][levelID].ptr == NULL )
-		    samp1[houroy][varID][levelID].ptr = malloc(gridsize*sizeof(double));
+		    samp1[houroy][varID][levelID].ptr = (double*) malloc(gridsize*sizeof(double));
 
 		  for ( i = 0; i < gridsize; i++ )
 		    if ( DBL_IS_EQUAL(vars1[houroy][varID][levelID].ptr[i],
@@ -203,7 +203,7 @@ void *Yhourstat(void *argument)
 		{
 		  if ( samp1[houroy][varID][levelID].ptr == NULL )
 		    {
-		      samp1[houroy][varID][levelID].ptr = malloc(gridsize*sizeof(double));
+		      samp1[houroy][varID][levelID].ptr = (double*) malloc(gridsize*sizeof(double));
 		      for ( i = 0; i < gridsize; i++ )
 			samp1[houroy][varID][levelID].ptr[i] = nsets[houroy];
 		    }
