@@ -48,6 +48,14 @@
 #include "math.h"
 #include "grid_cell.h"
 
+#ifndef  M_PI
+#define  M_PI        3.14159265358979323846264338327950288  /* pi */
+#endif
+
+#ifndef  M_PI_2
+#define  M_PI_2      1.57079632679489661923132169163975144  /* pi/2 */
+#endif
+
 #define EARTH_RADIUS 6371.2290
 
 static double const EarthRadius  = EARTH_RADIUS;
@@ -101,7 +109,7 @@ struct grid_vtable {
    void (*pack_grid)(struct grid *, double **, unsigned, unsigned *, unsigned *,
                      unsigned **, unsigned, unsigned *, unsigned *);
    struct grid_search * (*get_grid_search)(struct grid * grid);
-   void (*delete)(struct grid *);
+   void (*xdelete)(struct grid *);
 };
 
 struct grid {
@@ -458,7 +466,9 @@ struct grid * unpack_grid(double * dble_buf, unsigned * dble_buf_data_size,
 /**
  * generates a grid search object for the given grid
  * @param[in] grid
- * @return 
+ * @return grid_search object for the provided grid
+ * @remark the grid_search object returned by this routine is deleted by
+ *         \ref delete_grid
  */
 struct grid_search * get_grid_search(struct grid * grid);
 
